class MarkovGenerater
  def initialize(filename)
    @next = Hash.new
    @keys = Array.new
    @prefix = "\0"

    File.new(filename).each { |line|
      line.chomp!
      for i in 0...line.length/2
        add(line[i*2,2])
      end
      add("\0")
    }

    @prefix = @keys[rand(@keys.size)]
  end

  def add(str)
    if (@next[@prefix] == nil) then
      @next[@prefix] = [str]
      if (str != "\0") then
        @keys.push(@prefix)
      end
    else
      @next[@prefix].push(str)
    end
    @prefix = str
  end

  def generate
    ret = ""
    key = "\0"
    while (key == "\0")
      key = @keys[rand(@keys.size)]
      if (ENV['QUERY_STRING'] == 'negima') then key = '' end
    end
    while (key != "\0")
      ret += key
      n = @next[key]
      key = n[rand(n.size)]
    end

    return ret
  end

end

def spell_desc
  "spell>: ˡޤ"
end

def spell(msg)
  spellsFile = "data/spells.txt"
  if (ENV['QUERY_STRING'] == 'negima') then
    spellsFile = "data/spells_negima.txt"
  end

  markov = MarkovGenerater.new(spellsFile)
  spellname = markov.generate()

  adj = [
    '', '', '礭', '㤷', 'ۤȤФ', 'Ǯ', '',
    '䤹', '㤷', 'Ȥڤ$adj', 'Ƥڤ',
    '', 'Ƥ', '', '1ٹԤä', '夤'
  ]

  tar = [
    'ְ', '', 'Ũ', 'ޥ', '', '', '',
    'MP', 'ʬ', 'ˡ', 'ˡ', '', 'Хꥢ', 'ŴΤޤ',
    '᡼', 'Ȣ', '䤷'
  ]

  pref = [
    '$adjФζ̤', '$adj', '$adjȯ򵯤', 'ɹοϤ',
    '$adjε', '$adjʤ', 'Ϥ碌', 'θդꤲ',
    'ɥ饴ˤʤϤ', '̿ΤƼ', '̿ΤƤ',
    '', '1δ֤', ''
  ]

  suf = [
    '$randPtΥ᡼', '©κߤ', 'ѿȤƤ֤ϲ˺',
    'λޤƮǽˤ', 'HP$randPt', 'HP$adj',
    '$tarä', '$tarľ', '$tar1/2γΨ֤餻',
    '$tar֤餻', '$tar֤餻',
    '$tar󤯤ˤϤФ', '$tarä', '$tar򲼤',
    '$tarܤˤ', '$tarۤ',
    '$tarμ$tar$tarʬΤΤˤ',
    '$adj$tarä', '$tar夲', '$tar$adj̲Ͷ',
    '$tar', '$tar𤵤', '$tar$tar',
    '$tarϤ֤', '$tar$tar$tar',
    '$tarФ$tarϤ', '$tar$tarˤʤ',
    '$adjĮ¼˰ưǤ', 'ʪƶæФ',
    '$tar$adj$tarФʤʤ', '$tar᡼ʤʤ',
    '$tarȤ򸫶ˤ', '$tar$tarȯ', '뤫狼ʤ'
  ]

  msg = suf[rand(suf.size)]
  if (rand(2) < 1) then
    msg = pref[rand(pref.size)] + msg
  end

  while (msg.sub!(/\$adj/, adj[rand(adj.size)]) != nil ||
           msg.sub!(/\$tar/, tar[rand(tar.size)]) != nil ||
           msg.sub!(/\$rand/, (rand(27)*rand(27)).to_s))
  end

  [spellname, msg]
end
